/**
 * DevExtreme (esm/exporter/exceljs/export_pivot_grid.js)
 * Version: 21.1.0 (build 21075-0317)
 * Build date: Tue Mar 16 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import {
    isDefined,
    isObject
} from "../../core/utils/type";
import {
    Export
} from "./export";
import {
    noop
} from "../../core/utils/common";
var privateOptions = {
    _getWorksheetFrozenState: function(dataProvider, cellRange) {
        return {
            state: "frozen",
            xSplit: cellRange.from.column + dataProvider.getFrozenArea().x - 1,
            ySplit: cellRange.from.row + dataProvider.getFrozenArea().y - 1
        }
    },
    _getCustomizeCellOptions: function(excelCell, pivotCell) {
        return {
            excelCell: excelCell,
            pivotCell: pivotCell
        }
    },
    _needMergeRange: function() {
        return true
    },
    _renderLoadPanel: function(component) {
        component._renderLoadPanel(component._dataArea.groupElement(), component.$element())
    },
    _trySetAutoFilter: noop,
    _trySetFont: noop,
    _trySetOutlineLevel: noop
};

function exportPivotGrid(options) {
    return Export.export(_getFullOptions(options), privateOptions)
}

function _getFullOptions(options) {
    if (!(isDefined(options) && isObject(options))) {
        throw Error('The "exportPivotGrid" method requires a configuration object.')
    }
    if (!(isDefined(options.component) && isObject(options.component) && "dxPivotGrid" === options.component.NAME)) {
        throw Error('The "component" field must contain a PivotGrid instance.')
    }
    return Export.getFullOptions(options)
}
export {
    exportPivotGrid
};
