/**
 * DevExtreme (esm/ui/file_manager/ui.file_manager.notification.js)
 * Version: 21.1.0 (build 21075-0317)
 * Build date: Tue Mar 16 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import $ from "../../core/renderer";
import {
    extend
} from "../../core/utils/extend";
import {
    isFunction
} from "../../core/utils/type";
import {
    Deferred
} from "../../core/utils/deferred";
import {
    getWindow,
    hasWindow
} from "../../core/utils/window";
import Widget from "../widget/ui.widget";
import Popup from "../popup";
import Drawer from "../drawer/ui.drawer";
import FileManagerProgressPanel from "./ui.file_manager.notification.progress_panel";
var window = getWindow();
var ADAPTIVE_STATE_SCREEN_WIDTH = 1e3;
var FILE_MANAGER_NOTIFICATION_CLASS = "dx-filemanager-notification";
var FILE_MANAGER_NOTIFICATION_DRAWER_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-drawer");
var FILE_MANAGER_NOTIFICATION_DRAWER_PANEL_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_DRAWER_CLASS, "-panel");
var FILE_MANAGER_NOTIFICATION_POPUP_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-popup");
var FILE_MANAGER_NOTIFICATION_POPUP_ERROR_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-popup-error");
var FILE_MANAGER_NOTIFICATION_COMMON_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-common");
var FILE_MANAGER_NOTIFICATION_SEPARATOR_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-separator");
var FILE_MANAGER_NOTIFICATION_DETAILS_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-details");
var FILE_MANAGER_NOTIFICATION_COMMON_NO_ITEM_CLASS = "".concat(FILE_MANAGER_NOTIFICATION_CLASS, "-common-no-item");
export default class FileManagerNotificationControl extends Widget {
    _initMarkup() {
        super._initMarkup();
        this._initActions();
        this._actionProgressStatus = "default";
        this._operationInProgressCount = 0;
        this._failedOperationCount = 0;
        this._isInAdaptiveState = this._isSmallScreen();
        var $progressPanelContainer = this.option("progressPanelContainer");
        var $progressDrawer = $("<div>").addClass(FILE_MANAGER_NOTIFICATION_DRAWER_CLASS).appendTo($progressPanelContainer);
        $("<div>").addClass(FILE_MANAGER_NOTIFICATION_DRAWER_PANEL_CLASS).appendTo($progressDrawer);
        var drawerOptions = extend({
            opened: false,
            position: "right",
            template: container => this._ensureProgressPanelCreated(container)
        }, this._getProgressDrawerAdaptiveOptions());
        this._progressDrawer = this._createComponent($progressDrawer, Drawer, drawerOptions);
        var $drawerContent = $progressDrawer.find(".".concat(FILE_MANAGER_NOTIFICATION_DRAWER_PANEL_CLASS)).first();
        var contentRenderer = this.option("contentTemplate");
        if (isFunction(contentRenderer)) {
            contentRenderer($drawerContent)
        }
    }
    tryShowProgressPanel() {
        var promise = new Deferred;
        if ("default" === this._actionProgressStatus || this._isProgressDrawerOpened()) {
            return promise.resolve().promise()
        }
        setTimeout(() => {
            this._progressDrawer.show().done(promise.resolve);
            this._getNotificationPopup().hide();
            this._tryHideActionProgress()
        });
        return promise.promise()
    }
    addOperation(processingMessage, allowCancel, allowProgressAutoUpdate) {
        this._operationInProgressCount++;
        var operationInfo = this._progressPanel.addOperation(processingMessage, allowCancel, allowProgressAutoUpdate);
        this._updateActionProgress(processingMessage, "progress");
        return operationInfo
    }
    addOperationDetails(info, details, showCloseButton) {
        this._progressPanel.addOperationDetails(info, details, showCloseButton)
    }
    updateOperationItemProgress(operationInfo, itemIndex, itemProgress, commonProgress) {
        this._progressPanel.updateOperationItemProgress(operationInfo, itemIndex, itemProgress, commonProgress)
    }
    completeOperationItem(operationInfo, itemIndex, commonProgress) {
        this._progressPanel.completeOperationItem(operationInfo, itemIndex, commonProgress)
    }
    completeOperation(info, commonText, isError, statusText) {
        this._operationInProgressCount--;
        if (isError) {
            this._failedOperationCount++
        } else {
            this._showPopup(commonText)
        }
        this._progressPanel.completeOperation(info, commonText, isError, statusText);
        if (!this._isProgressDrawerOpened() || !this._tryHideActionProgress()) {
            var status = 0 === this._failedOperationCount ? "success" : "error";
            this._updateActionProgress("", status)
        }
    }
    completeSingleOperationWithError(operationInfo, errorInfo) {
        this._progressPanel.completeSingleOperationWithError(operationInfo, errorInfo.detailErrorText);
        this._notifyError(errorInfo)
    }
    addOperationDetailsError(operationInfo, errorInfo) {
        this._progressPanel.addOperationDetailsError(operationInfo, errorInfo.itemIndex, errorInfo.detailErrorText);
        this._notifyError(errorInfo)
    }
    _hideProgressPanel() {
        setTimeout(() => this._progressDrawer.hide())
    }
    _tryHideActionProgress() {
        if (0 === this._operationInProgressCount && 0 === this._failedOperationCount) {
            this._updateActionProgress("", "default");
            return true
        }
        return false
    }
    _updateActionProgress(message, status) {
        this._actionProgressStatus = status;
        this._raiseActionProgress(message, status)
    }
    _isSmallScreen() {
        if (!hasWindow()) {
            return false
        }
        return $(window).width() <= ADAPTIVE_STATE_SCREEN_WIDTH
    }
    _dimensionChanged(dimension) {
        if (!dimension || "height" !== dimension) {
            this._checkAdaptiveState()
        }
    }
    _checkAdaptiveState() {
        var oldState = this._isInAdaptiveState;
        this._isInAdaptiveState = this._isSmallScreen();
        if (this._progressDrawer && oldState !== this._isInAdaptiveState) {
            if (this._progressPanel) {
                this._progressPanel.$element().detach()
            }
            var options = this._getProgressDrawerAdaptiveOptions();
            this._progressDrawer.option(options)
        }
    }
    _getProgressDrawerAdaptiveOptions() {
        if (this._isInAdaptiveState) {
            return {
                openedStateMode: "overlap",
                shading: true,
                closeOnOutsideClick: true
            }
        } else {
            return {
                openedStateMode: "shrink",
                shading: false,
                closeOnOutsideClick: false
            }
        }
    }
    _ensureProgressPanelCreated(container) {
        if (!this._progressPanel) {
            var $progressPanelElement = $("<div>").appendTo(container);
            this._progressPanel = this._createComponent($progressPanelElement, this._getProgressPanelComponent(), {
                onOperationClosed: _ref => {
                    var {
                        info: info
                    } = _ref;
                    return this._onProgressPanelOperationClosed(info)
                },
                onOperationCanceled: _ref2 => {
                    var {
                        info: info
                    } = _ref2;
                    return this._raiseOperationCanceled(info)
                },
                onOperationItemCanceled: _ref3 => {
                    var {
                        item: item,
                        itemIndex: itemIndex
                    } = _ref3;
                    return this._raiseOperationItemCanceled(item, itemIndex)
                },
                onPanelClosed: () => this._hideProgressPanel()
            })
        } else {
            this._progressPanel.$element().appendTo(container)
        }
    }
    _getProgressPanelComponent() {
        return FileManagerProgressPanel
    }
    _notifyError(errorInfo) {
        this._showPopupError(errorInfo);
        this._updateActionProgress(errorInfo.commonErrorText, "error")
    }
    _onProgressPanelOperationClosed(info) {
        if (info.hasError) {
            this._failedOperationCount--;
            this._tryHideActionProgress()
        }
    }
    _isProgressDrawerOpened() {
        return this._progressDrawer.option("opened")
    }
    _showPopup(content, errorMode) {
        if (this._isProgressDrawerOpened()) {
            return
        }
        this._getNotificationPopup()._wrapper().toggleClass(FILE_MANAGER_NOTIFICATION_POPUP_ERROR_CLASS, !!errorMode);
        this._getNotificationPopup().option("contentTemplate", content);
        if (!this._getNotificationPopup().option("visible")) {
            this._getNotificationPopup().show()
        }
    }
    _showPopupError(errorInfo) {
        var $content = $("<div>");
        var $message = $("<div>").addClass(FILE_MANAGER_NOTIFICATION_COMMON_CLASS).text(errorInfo.commonErrorText);
        var $separator = $("<div>").addClass(FILE_MANAGER_NOTIFICATION_SEPARATOR_CLASS);
        $("<div>").appendTo($separator);
        var $details = $("<div>").addClass(FILE_MANAGER_NOTIFICATION_DETAILS_CLASS);
        if (errorInfo.item) {
            this._progressPanel.createErrorDetailsProgressBox($details, errorInfo.item, errorInfo.detailErrorText)
        } else {
            $message.addClass(FILE_MANAGER_NOTIFICATION_COMMON_NO_ITEM_CLASS);
            this._progressPanel.renderError($details, $separator, errorInfo.detailErrorText)
        }
        $content.append($message, $separator, $details);
        this._showPopup($content, true)
    }
    _getNotificationPopup() {
        if (!this._notificationPopup) {
            var $popup = $("<div>").addClass(FILE_MANAGER_NOTIFICATION_POPUP_CLASS).appendTo(this.$element());
            this._notificationPopup = this._createComponent($popup, Popup, {
                container: this.$element(),
                width: "auto",
                height: "auto",
                showTitle: false,
                dragEnabled: false,
                shading: false,
                visible: false,
                closeOnOutsideClick: true,
                animation: {
                    duration: 0
                },
                position: {
                    my: "right top",
                    at: "right top",
                    of: this.option("positionTarget"),
                    offset: "-10 -5"
                }
            })
        }
        return this._notificationPopup
    }
    _raiseActionProgress(message, status) {
        this._actions.onActionProgress({
            message: message,
            status: status
        })
    }
    _raiseOperationCanceled(info) {
        this._actions.onOperationCanceled({
            info: info
        })
    }
    _raiseOperationItemCanceled(item, index) {
        this._actions.onOperationItemCanceled({
            item: item,
            itemIndex: index
        })
    }
    _initActions() {
        this._actions = {
            onActionProgress: this._createActionByOption("onActionProgress"),
            onOperationCanceled: this._createActionByOption("onOperationCanceled"),
            onOperationItemCanceled: this._createActionByOption("onOperationItemCanceled")
        }
    }
    _getDefaultOptions() {
        return extend(super._getDefaultOptions(), {
            progressPanelContainer: null,
            contentTemplate: null,
            onActionProgress: null,
            onOperationCanceled: null,
            onOperationItemCanceled: null
        })
    }
    _optionChanged(args) {
        var name = args.name;
        switch (name) {
            case "progressPanelContainer":
            case "contentTemplate":
                break;
            case "onActionProgress":
            case "onOperationCanceled":
            case "onOperationItemCanceled":
                this._actions[name] = this._createActionByOption(name);
                break;
            default:
                super._optionChanged(args)
        }
    }
}
