/**
 * DevExtreme (esm/ui/scheduler/timezones/utils.timezones_data.js)
 * Version: 21.1.0 (build 21075-0317)
 * Build date: Tue Mar 16 2021
 *
 * Copyright (c) 2012 - 2021 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
import query from "../../../data/query";
import errors from "../../../core/errors";
import tzData from "./timezones_data";
import {
    sign
} from "../../../core/utils/math";
var timeZoneDataUtils = {
    _timeZones: tzData.zones,
    getDisplayedTimeZones: function(timestamp) {
        var timeZones = this._timeZones.map(timezone => {
            var offset = this.getUtcOffset(timezone.offsets, timezone.offsetIndices, timezone.untils, timestamp);
            var title = "(GMT ".concat(this.formatOffset(offset), ") ").concat(this.formatId(timezone.id));
            return {
                offset: offset,
                title: title,
                id: timezone.id
            }
        });
        return query(timeZones).sortBy("offset").toArray()
    },
    formatOffset: function(offset) {
        var hours = Math.floor(offset);
        var minutesInDecimal = offset - hours;
        var signString = sign(offset) >= 0 ? "+" : "-";
        var hoursString = "0".concat(Math.abs(hours)).slice(-2);
        var minutesString = minutesInDecimal > 0 ? ":".concat(60 * minutesInDecimal) : ":00";
        return signString + hoursString + minutesString
    },
    formatId: function(id) {
        return id.split("/").join(" - ").split("_").join(" ")
    },
    getTimezoneById: function(id) {
        var result;
        var i = 0;
        var tzList = this._timeZones;
        if (id) {
            while (!result) {
                if (!tzList[i]) {
                    errors.log("W0009", id);
                    return
                }
                var currentId = tzList[i].id;
                if (currentId === id) {
                    result = tzList[i]
                }
                i++
            }
        }
        return result
    },
    getTimeZoneOffsetById: function(id, timestamp) {
        var tz = this.getTimezoneById(id);
        if (tz) {
            return this.getUtcOffset(tz.offsets, tz.offsetIndices, tz.untils, timestamp)
        }
        return
    },
    getTimeZoneDeclarationTuple: function(id, year) {
        var tz = this.getTimezoneById(id);
        if (tz) {
            return this.getTimeZoneDeclarationTupleCore(tz.offsets, tz.offsetIndices, tz.untils, year)
        }
        return []
    },
    getTimeZoneDeclarationTupleCore: function(offsets, offsetIndices, untils, year) {
        var tupleResult = [];
        var offsetIndicesList = offsetIndices.split("");
        var offsetsList = offsets.split("|").map(value => parseInt(value));
        var untilsList = untils.split("|").map(until => {
            if ("Infinity" === until) {
                return null
            }
            return 1e3 * parseInt(until, 36)
        });
        var currentDate = 0;
        for (var i = 0, listLength = untilsList.length; i < listLength; i++) {
            currentDate += untilsList[i];
            if (new Date(currentDate).getFullYear() === year) {
                var offset = offsetsList[Number(offsetIndicesList[i + 1])];
                tupleResult.push({
                    date: currentDate,
                    offset: -offset / 60
                })
            }
            if (new Date(currentDate).getFullYear() > year) {
                break
            }
        }
        return tupleResult
    },
    getUtcOffset: function(offsets, offsetIndices, untils, dateTimeStamp) {
        var index = 0;
        var offsetIndicesList = offsetIndices.split("");
        var offsetsList = offsets.split("|");
        var untilsList = untils.split("|").map(until => {
            if ("Infinity" === until) {
                return null
            }
            return 1e3 * parseInt(until, 36)
        });
        var currentUntil = 0;
        for (var i = 0, listLength = untilsList.length; i < listLength; i++) {
            currentUntil += untilsList[i];
            if (dateTimeStamp >= currentUntil) {
                index = i;
                continue
            } else {
                break
            }
        }
        if (untilsList[index + 1]) {
            index++
        }
        var offset = Number(offsetsList[Number(offsetIndicesList[index])]);
        return -offset / 60 || offset
    }
};
export default timeZoneDataUtils;
